@@ -27,6 +27,7 @@
 # reasoning this is based on the MATLAB lexer:
 #   - increases complexity in the MATLAB lexer a bit, since it adds another
 #     lexing mode
+#   + we can re-use the error raising mechanism
 #   + we don't have to implement a separate lexer
 #   + we don't have to depend on e.g. PLY (which is not a bad thing, I love
 #     PLY, but since we have zero dependencies right now this would be
@@ -44,6 +45,7 @@ from miss_hit_core.config import (STYLE_RULES, STYLE_CONFIGURATION,
                                   Boolean_Style_Configuration,
                                   Integer_Style_Configuration,
                                   Regex_Style_Configuration,
+                                  Encoding_Style_Configuration,
                                   Set_Style_Configuration)
 from miss_hit_core.cfg_ast import *
 
@@ -54,7 +56,7 @@ class Config_Parser:
         assert isinstance(config_file, str)
 
         self.filename = config_file
-        self.dirname = os.path.dirname(config_file)
+        self.dirname = os.path.dirname(os.path.abspath(config_file))
         self.mh = mh
 
         self.mh.register_file(self.filename)
@@ -77,19 +79,9 @@ class Config_Parser:
         self.ct = self.nt
         self.nt = self.lexer.token()
 
-        while self.nt:
-            # Skip comments
-            while self.nt and self.nt.kind == "COMMENT":
-                self.nt = self.lexer.token()
-
-            # Join new-lines
-            if (self.nt and
-                self.ct and
-                self.nt.kind == "NEWLINE" and
-                self.ct.kind == "NEWLINE"):
-                self.nt = self.lexer.token()
-            else:
-                break
+        # Skip comments and newlines
+        while self.nt and self.nt.kind in ("COMMENT", "NEWLINE"):
+            self.nt = self.lexer.token()
 
     def match(self, kind, value=None):
         self.skip()
@@ -132,21 +124,18 @@ class Config_Parser:
         has_errors = False
 
         while not self.peek_eof():
-            if self.peek("NEWLINE"):
-                self.match("NEWLINE")
-            else:
-                try:
-                    n_item = self.parse_config_item()
-                except Error:
-                    # On errors, we skip tokens until we get to a
-                    # newline
-                    has_errors = True
-                    n_item = None
-                    while not (self.peek_eof() or self.peek("NEWLINE")):
-                        self.skip()
+            try:
+                n_item = self.parse_config_item()
+            except Error:
+                # On errors, we skip tokens until we get to a
+                # newline
+                has_errors = True
+                n_item = None
+                while not self.peek_eof() and not self.nt.first_in_line:
+                    self.skip()
 
-                if n_item:
-                    n_file.add_item(n_item)
+            if n_item:
+                n_file.add_item(n_item)
 
         self.match_eof()
 
@@ -179,6 +168,23 @@ class Config_Parser:
         elif self.peek("IDENTIFIER", "octave"):
             n_item = self.parse_octave_mode()
 
+        elif self.peek("IDENTIFIER", "entrypoint"):
+            n_item = self.parse_entrypoint()
+
+        elif self.peek("IDENTIFIER", "library"):
+            n_item = self.parse_library()
+
+        elif self.peek("KEYWORD", "global"):
+            self.match("KEYWORD", "global")
+            if self.peek("IDENTIFIER", "library"):
+                n_item = self.parse_library()
+                n_item.set_global()
+            else:
+                self.skip()
+                self.mh.error(self.ct.location,
+                              "expected library, found %s instead" %
+                              (self.ct.kind))
+
         elif self.peek("IDENTIFIER"):
             n_item = self.parse_style_configuration()
 
@@ -188,12 +194,6 @@ class Config_Parser:
                           "expected valid config entry, found %s instead" %
                           (self.ct.kind))
 
-        # Each item is terminated with a newline (or EOF)
-        if self.peek_eof():
-            pass
-        else:
-            self.match("NEWLINE")
-
         return n_item
 
     def parse_integer_number(self):
@@ -230,6 +230,17 @@ class Config_Parser:
 
         return value
 
+    def parse_encoding(self):
+        value = self.parse_string()
+
+        # We can use the Python regex module to sanity check it
+        try:
+            "potato".encode(value)
+        except LookupError:
+            self.mh.error(self.ct.location, "not a valid encoding")
+
+        return value
+
     def parse_boolean(self):
         if self.peek("NUMBER"):
             value = self.parse_integer_number()
@@ -317,6 +328,9 @@ class Config_Parser:
         elif isinstance(cfg_item, Regex_Style_Configuration):
             value = self.parse_regex()
 
+        elif isinstance(cfg_item, Encoding_Style_Configuration):
+            value = self.parse_encoding()
+
         elif isinstance(cfg_item, Set_Style_Configuration):
             value = self.parse_string()
 
@@ -403,6 +417,80 @@ class Config_Parser:
         value = self.parse_boolean()
         return Octave_Mode(value)
 
+    def parse_entrypoint(self):
+        self.match("IDENTIFIER", "entrypoint")
+        self.match("STRING")
+        rv = Entrypoint_Declaration(self.ct.location,
+                                    self.dirname,
+                                    self.ct.value)
+        self.match("C_BRA")
+
+        while not (self.peek("C_KET") or self.peek_eof()):
+            if self.peek("IDENTIFIER", "libraries"):
+                for t_lib in self.parse_lib_dependencies():
+                    rv.add_lib_dependency(self.mh, t_lib)
+            elif self.peek("IDENTIFIER", "paths"):
+                for t_path in self.parse_lib_paths():
+                    rv.add_path(self.mh, t_path)
+            else:
+                self.mh.error(self.nt.location,
+                              "expected libraries|paths property")
+
+        self.match("C_KET")
+        return rv
+
+    def parse_library(self):
+        self.match("IDENTIFIER", "library")
+        if self.peek("STRING"):
+            self.match("STRING")
+            name = self.ct.value
+        else:
+            name = None
+        rv = Library_Declaration(self.ct.location,
+                                 self.dirname,
+                                 name)
+
+        self.match("C_BRA")
+
+        while not (self.peek("C_KET") or self.peek_eof()):
+            if self.peek("IDENTIFIER", "paths"):
+                for t_path in self.parse_lib_paths():
+                    rv.add_path(self.mh, t_path)
+            else:
+                self.mh.error(self.nt.location,
+                              "expected paths property")
+
+        self.match("C_KET")
+        return rv
+
+    def parse_lib_dependencies(self):
+        rv = []
+        self.match("IDENTIFIER", "libraries")
+        self.match("C_BRA")
+        while self.peek("STRING"):
+            self.match("STRING")
+            rv.append(self.ct)
+            if self.peek("COMMA"):
+                self.match("COMMA")
+            else:
+                break
+        self.match("C_KET")
+        return rv
+
+    def parse_lib_paths(self):
+        rv = []
+        self.match("IDENTIFIER", "paths")
+        self.match("C_BRA")
+        while self.peek("STRING"):
+            self.match("STRING")
+            rv.append(self.ct)
+            if self.peek("COMMA"):
+                self.match("COMMA")
+            else:
+                break
+        self.match("C_KET")
+        return rv
+
 
 def load_config(mh, filename):
     cfg_parser = Config_Parser(mh, os.path.relpath(filename))
