@@ -100,6 +100,12 @@ class Location:
             rv["context"] = self.context
         return rv
 
+    def short_string(self):
+        rv = self.filename
+        if self.line:
+            rv += ":%u" % self.line
+        return rv
+
 
 class ICE(Exception):
     """ Internal compiler errors """
@@ -196,7 +202,7 @@ class Check_Message(Message):
 class Message_Handler:
     """ All messages should be routed through this class """
     def __init__(self, tool_id):
-        assert tool_id in ("debug", "style", "metric", "lint", "bmc")
+        assert tool_id in ("debug", "style", "metric", "lint", "bmc", "diff")
 
         self.tool_id = tool_id
 
@@ -545,6 +551,13 @@ class Message_Handler:
                       autofixed = False)
         self.register_message(msg)
 
+    def config_error(self, location, message):
+        # This is for raising errors in config files _only_.
+        canonical_filename = location.filename.replace("\\", "/")
+        if canonical_filename not in self.files:
+            self.register_file(location.filename)
+        self.error(location, message)
+
     def finalize_file(self, filename):
         assert isinstance(filename, str)
         canonical_filename = filename.replace("\\", "/")
@@ -565,6 +578,10 @@ class Message_Handler:
         # Remove file
         self.unregister_file(canonical_filename)
 
+    def command_line_error(self, message):
+        print("%s: error: %s" % (os.path.basename(sys.argv[0]), message))
+        sys.exit(1)
+
     def summary_and_exit(self):
         files = list(self.messages)
         for filename in files:
