@@ -29,6 +29,7 @@
 
 import os
 import re
+from copy import copy
 
 from abc import ABCMeta, abstractmethod
 
@@ -45,7 +46,7 @@ from miss_hit_core.m_lexer import MATLAB_Lexer, Token_Buffer
 from miss_hit_core.m_parser import MATLAB_Parser
 
 
-COPYRIGHT_REGEX = r"(\(c\) )?Copyright (\d\d\d\d-)?\d\d\d\d *(?P<org>.*)"
+COPYRIGHT_REGEX = r"(\(c\) )?Copyright (\d\d\d\d-)?\d\d\d\d( by)? *(?P<org>.*)"
 
 
 class Style_Rule(metaclass=ABCMeta):
@@ -606,16 +607,83 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
                                fixed)
                 token.fix.ensure_trim_before = True
 
-        # Corresponds to the old CodeChecker KeywordWhitespace rule
-        elif (token.kind == "KEYWORD" and
-              token.value in KEYWORDS_WITH_WS):
-            if cfg.active("whitespace_keywords") and \
+        elif token.kind == "KEYWORD":
+            # Corresponds to the old CodeChecker KeywordWhitespace rule
+            if token.value in KEYWORDS_WITH_WS and \
+               cfg.active("whitespace_keywords") and \
                next_in_line and ws_after == 0:
                 mh.style_issue(token.location,
                                "keyword must be succeeded by whitespace",
                                fixed)
                 token.fix.ensure_ws_after = True
 
+            # Make sure we have whitespace _before_ the function
+            # keyword
+            if token.value == "function" and \
+               cfg.active("whitespace_around_functions"):
+                # There is a special exception here for comments
+                # before functions
+                true_fstart_token = token
+                true_fstart_prev_token = prev_token
+                for i in reversed(range(0, n)):
+                    if tbuf.tokens[i].kind == "COMMENT" and \
+                       tbuf.tokens[i].first_in_line:
+                        true_fstart_token = tbuf.tokens[i]
+                        if i > 0:
+                            true_fstart_prev_token = tbuf.tokens[i - 1]
+                        else:
+                            true_fstart_prev_token = None
+                    elif tbuf.tokens[i].kind == "NEWLINE" and \
+                         tbuf.tokens[i].value.count("\n") == 1:
+                        pass
+                    else:
+                        break
+
+                if true_fstart_prev_token and \
+                   not (true_fstart_prev_token.location.line + 1 <
+                        true_fstart_token.location.line):
+                    true_fstart_prev_token.fix.add_newline = True
+                    mh.style_issue(token.location,
+                                   "function should be preceeded by an empty"
+                                   " line",
+                                   fixed)
+
+            # Make sure we have whitespace _after_ the function end
+            elif token.value == "end" and \
+                 cfg.active("whitespace_around_functions") and \
+                 isinstance(token.ast_link, Function_Definition):
+                # We first need to find the actual last token on this line
+                true_end_id = n
+                true_end_next = None
+                true_end_nl = None
+                for i in range(n + 1, len(tbuf.tokens)):
+                    if tbuf.tokens[i].kind == "NEWLINE":
+                        true_end_nl = tbuf.tokens[i]
+                        break
+                    elif tbuf.tokens[i].location.line == token.location.line:
+                        true_end_id = i
+                    else:
+                        break
+                true_end = tbuf.tokens[true_end_id]
+                for i in range(true_end_id + 1, len(tbuf.tokens)):
+                    if tbuf.tokens[i].kind == "NEWLINE":
+                        pass
+                    else:
+                        true_end_next = tbuf.tokens[i]
+                        break
+
+                if true_end_next and \
+                   not (true_end.location.line + 1 <
+                        true_end_next.location.line):
+                    mh.style_issue(token.location,
+                                   "function should be suceeded by an"
+                                   " empty line",
+                                   fixed)
+                    if true_end_nl:
+                        true_end_nl.fix.add_newline = True
+                    else:
+                        true_end.fix.add_newline = True
+
         # Corresponds to the old CodeChecker CommentWhitespace rule
         elif token.kind == "COMMENT":
             if cfg.active("whitespace_comments"):
@@ -875,6 +943,20 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
         elif token.kind in ("M_KET", "C_KET", "KET"):
             bracket_stack.pop()
 
+        # Finally, check for unicode problems.
+        if cfg.active("unicode") and \
+           (cfg.style_config["enforce_encoding_comments"] or
+            token.kind not in ("COMMENT", "CONTINUATION")):
+            try:
+                token.raw_text.encode(cfg.style_config["enforce_encoding"])
+            except UnicodeEncodeError as uee:
+                new_location = copy(token.location)
+                new_location.col_start = token.location.col_start + uee.start
+                new_location.col_end = token.location.col_start + (uee.end - 1)
+                mh.style_issue(new_location,
+                               "non-%s character in source" %
+                               cfg.style_config["enforce_encoding"])
+
 
 class MH_Style_Result(work_package.Result):
     def __init__(self, wp):
